@@ -9,6 +9,7 @@ import logging
 import os
 import platform as _platform
 import posixpath
+import shutil
 import subprocess
 import sys
 import time
@@ -51,6 +52,7 @@ from calkit.cli.new import new_app
 from calkit.cli.notebooks import notebooks_app
 from calkit.cli.office import office_app
 from calkit.cli.overleaf import overleaf_app
+from calkit.cli.slurm import slurm_app
 from calkit.cli.update import update_app
 from calkit.environments import get_env_lock_fpath
 from calkit.models import Procedure
@@ -77,6 +79,7 @@ app.add_typer(update_app, name="update", help="Update objects.")
 app.add_typer(check_app, name="check", help="Check things.")
 app.add_typer(overleaf_app, name="overleaf", help="Interact with Overleaf.")
 app.add_typer(cloud_app, name="cloud", help="Interact with a Calkit Cloud.")
+app.add_typer(slurm_app, name="slurm", help="Work with SLURM.")
 
 # Constants for version control auto-ignore
 AUTO_IGNORE_SUFFIXES = [".DS_Store", ".env", ".pyc", ".synctex.gz"]
@@ -187,8 +190,11 @@ def clone(
     no_dvc_pull: Annotated[
         bool, typer.Option("--no-dvc-pull", help="Do not pull DVC objects.")
     ] = False,
-    recursive: Annotated[
-        bool, typer.Option("--recursive", help="Recursively clone submodules.")
+    non_recursive: Annotated[
+        bool,
+        typer.Option(
+            "--no-recursive", help="Do not recursively clone submodules."
+        ),
     ] = False,
 ):
     """Clone a Git repo and by default configure and pull from the DVC
@@ -219,7 +225,7 @@ def clone(
             url = url.replace("https://github.com/", "git@github.com:")
     # Git clone
     cmd = ["git", "clone", url]
-    if recursive:
+    if not non_recursive:
         cmd.append("--recursive")
     if location is not None:
         cmd.append(location)
@@ -552,6 +558,15 @@ def save(
             help="Additional DVC args to pass when pushing.",
         ),
     ] = [],
+    no_recursive: Annotated[
+        bool, typer.Option("--no-recursive", help="Do not push to submodules.")
+    ] = False,
+    sync_overleaf: Annotated[
+        bool,
+        typer.Option(
+            "--overleaf", "-O", help="Sync with Overleaf after saving."
+        ),
+    ] = False,
     verbose: Annotated[
         bool, typer.Option("--verbose", "-v", help="Print verbose output.")
     ] = False,
@@ -572,6 +587,9 @@ def save(
             typer.echo(
                 f"Generating commit message for staged files: {staged_files}"
             )
+        if not staged_files:
+            typer.echo("No changes to commit; exiting")
+            raise typer.Exit(0)
         if len(staged_files) != 1:
             raise_error(
                 "Automatic commit messages can only be generated when "
@@ -595,19 +613,27 @@ def save(
         message = typer.prompt("Message")
     # Figure out if we have any DVC files in this commit, and if not, we can
     # skip pushing to DVC
+    staged_files = calkit.git.get_staged_files()
+    if not staged_files:
+        typer.echo("No changes to commit; exiting")
+        raise typer.Exit(0)
     any_dvc = any(
-        [
-            path == "dvc.lock" or path.endswith(".dvc")
-            for path in calkit.git.get_staged_files()
-        ]
+        [path == "dvc.lock" or path.endswith(".dvc") for path in staged_files]
     )
     commit(all=True if paths is None else False, message=message)
     if not no_push:
         if verbose and not any_dvc:
             typer.echo("Not pushing to DVC since no DVC files were staged")
         push(
-            no_dvc=not any_dvc, git_args=git_push_args, dvc_args=dvc_push_args
+            no_dvc=not any_dvc,
+            git_args=git_push_args,
+            dvc_args=dvc_push_args,
+            no_recursive=no_recursive,
         )
+    if sync_overleaf:
+        from calkit.cli.overleaf import sync as overleaf_sync
+
+        overleaf_sync(verbose=verbose, no_push=no_push)
 
 
 @app.command(name="pull")
@@ -629,6 +655,12 @@ def pull(
             help="Force pull, potentially overwriting local changes.",
         ),
     ] = False,
+    no_recursive: Annotated[
+        bool,
+        typer.Option(
+            "--no-recursive", help="Do not recursively pull from submodules."
+        ),
+    ] = False,
 ):
     """Pull with both Git and DVC."""
     typer.echo("Git pulling")
@@ -638,7 +670,10 @@ def pull(
         if "-f" not in dvc_args and "--force" not in dvc_args:
             dvc_args.append("-f")
     try:
-        subprocess.check_call(["git", "pull"] + git_args)
+        git_cmd = ["git", "pull"]
+        if not no_recursive and "--recurse-submodules" not in git_args:
+            git_cmd.append("--recurse-submodules")
+        subprocess.check_call(git_cmd + git_args)
     except subprocess.CalledProcessError:
         raise_error("Git pull failed")
     typer.echo("DVC pulling")
@@ -667,11 +702,17 @@ def push(
         list[str],
         typer.Option("--dvc-arg", help="Additional DVC args."),
     ] = [],
+    no_recursive: Annotated[
+        bool, typer.Option("--no-recursive", help="Do not push to submodules.")
+    ] = False,
 ):
     """Push with both Git and DVC."""
     typer.echo("Pushing to Git remote")
     try:
-        subprocess.check_call(["git", "push"] + git_args)
+        git_cmd = ["git", "push"]
+        if not no_recursive and "--recurse-submodules" not in git_args:
+            git_cmd.append("--recurse-submodules=on-demand")
+        subprocess.check_call(git_cmd + git_args)
     except subprocess.CalledProcessError:
         raise_error("Git push failed")
     if not no_dvc:
@@ -981,22 +1022,23 @@ def run(
         except Exception as e:
             os.environ.pop("CALKIT_PIPELINE_RUNNING", None)
             raise_error(f"Pipeline compilation failed: {e}")
-    # Get status of Git repo before running
-    repo = git.Repo()
-    git_rev = repo.head.commit.hexsha
-    try:
-        git_branch = repo.active_branch.name
-    except TypeError:
-        # If no branch is checked out, we are in a detached HEAD state
-        git_branch = None
-    git_changed_files_before = calkit.git.get_changed_files(repo=repo)
-    git_staged_files_before = calkit.git.get_staged_files(repo=repo)
-    git_untracked_files_before = calkit.git.get_untracked_files(repo=repo)
-    # Get status of DVC repo before running
-    dvc_repo = dvc.repo.Repo()
-    dvc_status_before = dvc_repo.status()
-    dvc_data_status_before = dvc_repo.data_status()
-    dvc_data_status_before.pop("git", None)  # Remove git status
+    if save_log:
+        # Get status of Git repo before running
+        repo = git.Repo()
+        git_rev = repo.head.commit.hexsha
+        try:
+            git_branch = repo.active_branch.name
+        except TypeError:
+            # If no branch is checked out, we are in a detached HEAD state
+            git_branch = None
+        git_changed_files_before = calkit.git.get_changed_files(repo=repo)
+        git_staged_files_before = calkit.git.get_staged_files(repo=repo)
+        git_untracked_files_before = calkit.git.get_untracked_files(repo=repo)
+        # Get status of DVC repo before running
+        dvc_repo = dvc.repo.Repo()
+        dvc_status_before = dvc_repo.status()
+        dvc_data_status_before = dvc_repo.data_status()
+        dvc_data_status_before.pop("git", None)  # Remove git status
     if targets is None:
         targets = []
     args = deepcopy(targets)
@@ -1791,20 +1833,41 @@ def set_env_var(
 @app.command(name="upgrade")
 def upgrade():
     """Upgrade Calkit."""
-    if calkit.check_dep_exists("pipx"):
-        cmd = ["pipx", "upgrade", "calkit-python"]
-    elif calkit.check_dep_exists("uv"):
+    # First detect how Calkit is installed
+    # If installed with uv tool, calkit will be located at something like
+    # ~/.local/bin/calkit
+    which_calkit = shutil.which("calkit")
+    if which_calkit is None:
+        raise_error("Calkit is not installed")
+    split_path = os.path.normpath(str(which_calkit)).split(os.sep)
+    if (
+        ".local" in split_path
+        and "bin" in split_path
+        and calkit.check_dep_exists("uv")
+    ):
+        # This is a uv tool install
         cmd = [
             "uv",
-            "pip",
+            "tool",
             "install",
-            "--system",
             "--upgrade",
             "calkit-python",
         ]
+    elif "pipx" in split_path and calkit.check_dep_exists("pipx"):
+        cmd = ["pipx", "upgrade", "calkit-python"]
     else:
-        cmd = ["pip", "install", "--upgrade", "calkit-python"]
-    subprocess.run(cmd)
+        cmd = [
+            sys.executable,
+            "-m",
+            "pip",
+            "install",
+            "--upgrade",
+            "calkit-python",
+        ]
+    res = subprocess.run(cmd)
+    if res.returncode != 0:
+        raise_error("Upgrade failed")
+    typer.echo("Success!")
 
 
 @app.command(name="switch-branch")
@@ -1855,3 +1918,51 @@ def run_jupyter(
     """Run a command with the Jupyter CLI."""
     process = subprocess.run([sys.executable, "-m", "jupyter"] + sys.argv[2:])
     sys.exit(process.returncode)
+
+
+@app.command(name="latexmk")
+def run_latexmk(
+    tex_file: Annotated[str, typer.Argument(help="The .tex file to compile.")],
+    environment: Annotated[
+        str | None,
+        typer.Option(
+            "--env",
+            "-e",
+            help=("Environment in which to run latexmk, if applicable."),
+        ),
+    ] = None,
+):
+    """Compile a LaTeX document with latexmk.
+
+    If a Calkit environment is not specified, latexmk will be run in the
+    system environment if available. If not available, a TeX Live Docker
+    container will be used.
+    """
+    latexmk_cmd = [
+        "latexmk",
+        "-pdf",
+        "-cd",
+        "-silent",
+        "-synctex=1",
+        "-interaction=nonstopmode",
+        tex_file,
+    ]
+    if environment is not None:
+        cmd = ["calkit", "xenv", "--name", environment] + latexmk_cmd
+    elif calkit.check_dep_exists("latexmk"):
+        cmd = latexmk_cmd
+    else:
+        cmd = [
+            "docker",
+            "run",
+            "--rm",
+            "-v",
+            f"{os.getcwd()}:/work",
+            "-w",
+            "/work",
+            "texlive/texlive:latest-full",
+        ] + latexmk_cmd
+    try:
+        subprocess.check_call(cmd)
+    except subprocess.CalledProcessError:
+        raise_error("latexmk failed")
